@@ -8,19 +8,23 @@ import FactorTest.FactorTestMain as FM
 from tqdm import tqdm
 import pandas as pd
 if __name__=='__main__':
-    FB.DataRenewTime()
-    datainfo=pd.read_excel(DataInfopath)
-    for func in datainfo['函数'].unique():
-        getattr(FactorTest.UpdateRawData,func)(datainfo[datainfo['函数']==func])
-        print(func,'更新完成')
-    FB.DataRenewTime()
-    print('数据库部分已更新完成')
+    # FB.DataRenewTime()
+    # datainfo=pd.read_excel(DataInfopath)
+    # for func in datainfo['函数'].unique():
+    #     getattr(FactorTest.UpdateRawData,func)(datainfo[datainfo['函数']==func])
+    #     print(func,'更新完成')
+    # FB.DataRenewTime()
+    # print('数据库部分已更新完成')
     #因子库更新
     FB.FactorRenewTime()
     datainfo=pd.read_excel(FactorInfopath)
     for func in tqdm(datainfo['函数'].unique()):
-       getattr(FactorTest.UpdateFactorData,func)(datainfo[datainfo['函数']==func])
-       print(func,'更新完成')
+        try:
+            getattr(FactorTest.UpdateFactorData,func)(datainfo[datainfo['函数']==func])
+            print(func,'更新完成')
+        except:
+            print(func,'更新失败')
+
     
     FB.calcAllFactorAns()
     FB.FactorRenewTime()
