@@ -346,6 +346,7 @@ def getIndexComponent(indexname='wind'):
 def setStockPool(DF,DFfilter):
     """
     功能：将DFfilter中的股票在DF中筛选出来
+    DFfilter——筛选的股票，可以用getIndexComponent(stockpool)
     stockpool = ["000300.SH",'000905.SH','000985.SH']
     返回：筛选后的DF  columns=[code,date,xx]
     """
@@ -1011,11 +1012,20 @@ def calcIC(mer, factor_name):
     输入：包含因子值和回报的数据dataframe，因子名称
     输出：包含IC,ICIR,rankIC,rankICIR,t_value的序列
     '''
-    sp1 = mer.groupby('time').apply(lambda x: stats.pearsonr(x['ret'], x[factor_name])[0])
+    def calcpearsonr(x,f1,f2):
+        if(x.shape[0]<=2):
+            return np.nan
+        return stats.pearsonr(x[f1], x[f2])[0]
+    def calcspearmanr(x,f1,f2):
+        if(x.shape[0]<=2):
+            return np.nan
+        return stats.spearmanr(x[f1], x[f2])[0]
+    
+    sp1 = mer.groupby('time').apply(calcpearsonr,'ret',factor_name)
     IC = sp1.mean()
     ICIR = (sp1.mean()) / sp1.std() * 12 ** 0.5
    
-    sp2 = mer.groupby('time').apply(lambda x: stats.spearmanr(x['ret'], x[factor_name])[0])
+    sp2 = mer.groupby('time').apply(calcspearmanr,'ret',factor_name)
     rankIC = sp2.mean()
     rankICIR = (sp2.mean()) / sp2.std() * 12 ** 0.5
     t_v = sp2.mean() / sp2.std() * (sp2.count() - 1) ** 0.5
@@ -1148,19 +1158,22 @@ def calcAllFactorAns():
     import FactorTest.FactorTestMain as FM
     datainfo=pd.read_excel(FactorInfopath)
     for i in tqdm(datainfo.index):
-       Test=FM.FactorTest()
-       Test.getFactor(read_feather(Factorpath+datainfo.loc[i,'地址']))
-       Test.autotest(datainfo.loc[i,'因子名称'],asc=False)
-       datainfo.loc[i,'IC']=Test.ICAns[datainfo.loc[i,'因子名称']]['IC:']
-       datainfo.loc[i,'ICIR']=Test.ICAns[datainfo.loc[i,'因子名称']]['ICIR:']
-       FacDF=Test.portfolioList[datainfo.loc[i,'因子名称']]
-       if(datainfo.loc[i,'IC']<0):
-           FacDF['多空组合']=FacDF['多空组合']*-1
-       datainfo.loc[i,'多空年化收益']=calcPortfolioRet(FacDF['多空组合'])['年化收益率:']
-       datainfo.loc[i,'多空信息比率']=calcPortfolioRet(FacDF['多空组合'])['信息比率:']
-       FacDF['IC']=Test.ICList[datainfo.loc[i,'因子名称']]
-       FacDF.columns=pd.Series(FacDF.columns).apply(lambda x:str(x))
-       FacDF.reset_index().to_feather(Factorpath+'plotdata/'+datainfo.loc[i,'因子名称']+'.fth')
+        try:
+            Test=FM.FactorTest()
+            Test.getFactor(read_feather(Factorpath+datainfo.loc[i,'地址']))
+            Test.autotest(datainfo.loc[i,'因子名称'],startMonth=201001,asc=False)
+            datainfo.loc[i,'IC']=Test.ICAns[datainfo.loc[i,'因子名称']]['IC:']
+            datainfo.loc[i,'ICIR']=Test.ICAns[datainfo.loc[i,'因子名称']]['ICIR:']
+            FacDF=Test.portfolioList[datainfo.loc[i,'因子名称']]
+            if(datainfo.loc[i,'IC']<0):
+                FacDF['多空组合']=FacDF['多空组合']*-1
+            datainfo.loc[i,'多空年化收益']=calcPortfolioRet(FacDF['多空组合'])['年化收益率:']
+            datainfo.loc[i,'多空信息比率']=calcPortfolioRet(FacDF['多空组合'])['信息比率:']
+            FacDF['IC']=Test.ICList[datainfo.loc[i,'因子名称']]
+            FacDF.columns=pd.Series(FacDF.columns).apply(lambda x:str(x))
+            FacDF.reset_index().to_feather(Factorpath+'plotdata/'+datainfo.loc[i,'因子名称']+'.fth')
+        except:
+            print('datainfo.loc[i,"因子名称"]有误')
     datainfo.to_excel(FactorInfopath, index=False)
 
 def calcPortfolioRet(Rev_seq,t=12):
@@ -1512,6 +1525,97 @@ def transFisicalData(data,key='factor',ANN_DT='BasicFactor_AShareFinancialIndica
     return transData(data,key,ANN_DT,output,startmonth,endmonth)
 
 
+def transPriceData_Month(data,key='factor',ANN_DT='BasicFactor_AShareFinancialIndicator_ANN_DT.txt', output='', startmonth=199912, endmonth=210012):
+    '''
+    功能：针对月度量价数据的日期转换，例如公告日当月的收益率
+    data为因子矩阵,需预先getFisicalList()处理
+    ANN_DT为公布日期文件地址，默认'BasicFactor_AShareFinancialIndicator_ANN_DT.txt'；可选三大表ANN_DT
+    '''
+    monthlist=getTradeDateList('month')
+
+    ANN_DT='BasicFactor_AShareFinancialIndicator_ANN_DT.txt'
+    ANN_DT=pd.read_feather(Datapath + ANN_DT).set_index('time').reindex(getFisicalList())
+    ANN_DT.index.name='time'
+    ANN_DT.columns.name='code'
+    ANN_DT = ANN_DT.stack().reset_index().set_index('code').astype(int).reset_index()
+    
+    if('time' in data.columns):
+        df_temp = data.drop(['time'],axis=1).stack().reset_index()
+    else:
+        df_temp = data.stack().reset_index()
+    df_temp.columns = ['time','code',key]
+    df_temp['month'] = df_temp['time']//100
+    ANN_DT[1] = ANN_DT[0]//100
+    ANN_DT.columns = ['code','time','ann_date','month']
+    Mer = ANN_DT.merge(df_temp,on=['code','month'],how='outer').dropna()[['code','month',key]].drop_duplicates()
+    Mer.columns = ['code','time',key]
+    Mer = Mer[['time','code',key]]
+    Mer = Mer.pivot(index='time',columns='code',values=key).fillna(method='ffill')
+    Mer = Mer.stack().reset_index()
+    Mer.columns = ['time','code',key]
+    
+    Mer = Mer[Mer.time >= startmonth]
+    Mer = Mer[Mer.time <= endmonth]
+    
+    return Mer
+
+def transPriceData_Day(data,key='factor',adj='',ANN_DT='BasicFactor_AShareFinancialIndicator_ANN_DT.txt', output='', startmonth=199912, endmonth=210012):
+    '''
+    功能：针对日度量价数据的日期转换，例如公告日前后三天收益率
+    数据要求：data的index必须为日期（yyyymmdd）
+    adj='pre'代表将非交易日的公告日期转换为日期小于这一天的最晚交易日；adj='next'代表将非交易日的公告日期转换为日期大于这一天的最早交易日
+    adj默认为'pre'
+    data为因子矩阵,需预先getFisicalList()处理
+    ANN_DT为公布日期文件地址，默认'BasicFactor_AShareFinancialIndicator_ANN_DT.txt'；可选三大表ANN_DT
+    '''
+    if(adj==''):
+        adj = 'pre'
+    monthlist=getTradeDateList('month')
+
+    ANN_DT='BasicFactor_AShareFinancialIndicator_ANN_DT.txt'
+    ANN_DT=pd.read_feather(Datapath + ANN_DT).set_index('time').reindex(getFisicalList())
+    ANN_DT.index.name='time'
+    ANN_DT.columns.name='code'
+    ANN_DT = ANN_DT.stack().reset_index().set_index('code').astype(int).reset_index()
+    
+    Trade_Date = getTradeDateList()
+    ANN_DT = ANN_DT[ANN_DT['time']>=Trade_Date.iloc[0]]
+    
+    ay_time = list(set(ANN_DT[0]))
+    ay_time_adj = []
+    for time in ay_time:
+        if(adj=='pre'):
+            ay_time_adj.append(Trade_Date[Trade_Date.values<=time].iloc[-1])
+        else:
+            ay_time_adj.append(Trade_Date[Trade_Date.values>=time].iloc[0])
+    dict1 = dict(zip(ay_time,ay_time_adj))
+    ANN_DT[0] = ANN_DT[0].apply(lambda x:dict1.get(x))
+    
+    if('time' in data.columns):
+        df_temp = data.drop(['time'],axis=1).stack().reset_index()
+    else:
+        df_temp = data.stack().reset_index()
+
+    ANN_DT.columns = ['code','time','ann_date']
+    df_temp.columns = ['ann_date','code',key]
+    
+    Mer = ANN_DT.merge(df_temp,on=['code','ann_date'],how='outer').dropna()
+    Mer = Mer[['ann_date','code',key]]
+    Mer.columns = ['time','code',key]
+    Mer.time = Mer.time//100
+    Mer = Mer.drop_duplicates(subset=['time','code'],keep='last')
+    Mer = Mer.pivot(index='time',columns='code',values=key)
+    monthlist = getTradeDateList('month')
+    Mer = Mer.reindex(monthlist[monthlist>=Mer.index[0]]).ffill()
+    Mer = Mer.stack().reset_index()
+    Mer.columns = ['time','code',key]
+    
+    Mer = Mer[Mer.time >= startmonth]
+    Mer = Mer[Mer.time <= endmonth]
+    
+    return Mer
+
+
 def calc_plot(DF):
     '''
     功能：对dataframe进行plot画图
